//#define DEBUGMAIN
//#define VISUALSTUDIO

#ifndef VISUALSTUDIO
	#include <ME218_C32.h>
	#include <TIMERS12.h>
#endif

#ifdef VISUALSTUDIO
	#include "ME218_C32.h"
	#include "TIMERS12.h"
#endif

#ifdef DEBUGMAIN
	#include <stdio.h>
#endif

#include "DigitalIO.h"
#include "CustomTime.h"
#include "Swag.h"
#include "PrizeLED.h"
#include "Stepper.h"
#include "CheckSlot.h"
#include "ADS12.h"
#include "GameTimerLED.h"

void main(void)
{
	//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	//$$$$            Declare Variables              $$$$
	//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//Declare Inputs
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	InputT CoinSensor;
	InputT DropButton;
	InputT LeftToggle;
	InputT RightToggle;
	InputT RightLimit;
	InputT LeftLimit;
	InputT Encoder;

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//Declare Outputs
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	OutputT PlaySound;
	OutputT Stepper;
	OutputT SwagOpen;
	OutputT SwagClose;
	OutputT PrizeLEDClock;
	OutputT PrizeLEDData;
	OutputT GameTimerLEDClock;
	OutputT GameTimerLEDData;
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//Declare Timers
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	//TrackTimer will manage how often we run stepper tracking
	CustomTimer TrackTimer;
	//GameTimer will manage how long we have to make a selection
	CustomTimer GameTimer;
	//LeftToggleDebounceTimer will manage debounce timing of the left toggle button.
	CustomTimer LeftToggleDebounceTimer;
	//RightToggleDebounceTimer will manage debounce timing of the right toggle button.
	CustomTimer RightToggleDebounceTimer;
	//SoundPlayTimer will monitor how long the sound should have been played so we can turn it off early.
	CustomTimer SoundPlayTimer;
	//GameTimerLED will manage the timing of the countdown Game Timer LED's.
	CustomTimer GameTimerLED;
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	//CurrentTime will be used for performing timer logic
	unsigned int CurrentTime;
	//leftlatch will be used for running code only once per button push
	unsigned char leftlatch=0;
	//rightlatch will be used for running code only once per button push
	unsigned char rightlatch=0;
	//InitReturn will be used for evaluating whether the Analog library was configured correctly.
	static ADS12ReturnTyp InitReturn;
	//EncoderResult is used for evaluating where the penny dropped.
	unsigned int EncoderResult;

	//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	//$$$$           Initialize Variables            $$$$
	//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//Initialize timer delays
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	//Set TrackTimer.Delay to 7 = 14ms.
	TrackTimer.Delay=7;
	//Set LeftToggleDebounceTimer.Delay and RightToggleDebounceTimer.Delay to 25 = 50ms.
	LeftToggleDebounceTimer.Delay=25;
	RightToggleDebounceTimer.Delay=25;
	//The Sound plays for approximately 20seconds.  Set SoundPlayTimer.Delay<20 seconds so we don't retrigger it.  9500=19seconds.
	SoundPlayTimer.Delay=9500;
	//Set GameTimer.Delay 14000=28seconds
	GameTimer.Delay=14000;
	//Set GameTimerLED.Delay 1666=3.333 seconds
	GameTimerLED.Delay=1666;
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//Initialize IO Variables
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	//Configure the coin sensor input at port M, bit 4.
	CoinSensor=CreatePortMInput(0x10);
	//Configure the drop button input at port AD, bit 5.
	DropButton=CreatePortADInput(0x20);
	//Configure the left toggle button input at port E, bit 0.
	LeftToggle=CreatePortEInput(0x01);
	//Configure the right toggle button input at port E, bit 1.
	RightToggle=CreatePortEInput(0x02);
	//Configure the sound play output at port T, bit 0.
	PlaySound=CreatePortTOutput(0x01);
	//Configure the right limit switch input at port T, bit 3.
	RightLimit=CreatePortTInput(0x08);
	//Configure the left limit switch input at port T, bit 2.
	LeftLimit=CreatePortTInput(0x04);
	//Configure the stepper outputs at port T, bits 4-7.
	Stepper=CreatePortTOutput(0xF0);
	//Configure the encoder inputs at port AD, bits 1-4.
	Encoder=CreatePortADInput(0x1E);
	//Configure the swag open output at port M, bit 2.
	SwagOpen=CreatePortMOutput(0x04);
	//Configure the swag closeoutput at port M, bit 3.
	SwagClose=CreatePortMOutput(0x08);
	//Configure the prize LED clock output at port M, bit 0.
	PrizeLEDClock=CreatePortMOutput(0x01);
	//Configure the prize LED data output at port M, bit 0.
	PrizeLEDData=CreatePortMOutput(0x02);
	//Configure the game timer LED clock output at port AD, bit 6.
	GameTimerLEDClock=CreatePortADOutput(0x40);
	//Configure the game timer LED data output at port AD, bit 7.
	GameTimerLEDData=CreatePortADOutput(0x80);

	//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	//$$$$           Initialize Libraries            $$$$
	//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

	//Initialize analog library so that pin AD0 is analog, AD1-AD5 is input, AD6-AD7 is output.
	InitReturn = ADS12_Init("OOIIIIIA");
	//Initialize TIMERS12 library.
	TMRS12_Init(TMRS12_RATE_2MS);
	//Initialize Stepper library.
	StepperInit(RightLimit, LeftLimit, Stepper);
	//Initialize Swag library.
	SwagInit(SwagOpen,SwagClose); 
	//Initialize PrizeLED libary.
	PrizeLEDInit(PrizeLEDClock, PrizeLEDData);
	//Initialize GameTimerLED library.
	GameTimerInit(GameTimerLEDClock, GameTimerLEDData);
	
	//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	//$$$$                Game Logic                 $$$$
	//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

	while(1){
		//Set Playsound output high.
		OutputWrite(PlaySound,0xff);
		//Reset Stepper to right limit switch
		StepperReset();
		//Reset prize LEDs
		PrizeLEDReset();
		//Reset game timer LEDs
		GameTimerReset();

		//wait for coin sensor to see the coin
		while(InputRead(CoinSensor)==0); 
		//Reset prize LEDs
		PrizeLEDReset();
		//Reset game timer LEDs
		GameTimerReset();
		//Play Sound
		OutputWrite(PlaySound,0x00);
		//Wait 200 ms to allow coin to fully fall into dropper before proceeding
		Wait(100);
		//Set sound play output high after it is triggered.
		OutputWrite(PlaySound,0xff);

		//Reset timers to current time
		TrackTimer=TimerUpdate(TrackTimer,TMRS12_GetTime());
		GameTimer=TimerUpdate(GameTimer,TrackTimer.StoredTime);
		SoundPlayTimer=TimerUpdate(SoundPlayTimer,TrackTimer.StoredTime);
		GameTimerLED=TimerUpdate(GameTimerLED,TrackTimer.StoredTime);

		//Begin non-blocking loop.  All code within may not use Wait().
		//While the drop button has not been pressed, do the following
		while(InputRead(DropButton)==0) 
		{
			//Set CurrentTime equal to the current time using TMRS12_GetTime()
			CurrentTime=TMRS12_GetTime();

			//if the tracking timer has expired 
			if(TimerDone(TrackTimer,CurrentTime))
			{
				//execute stepper tracking
				StepperTrack();
				//update the timer with CurrentTime
				TrackTimer=TimerUpdate(TrackTimer,CurrentTime);
			}			 
		
			//if the GameTimerLED timer has expired
			if(TimerDone(GameTimerLED,CurrentTime))
			{
				//count down on the LED's by 1
				GameTimerCountDown();
				//update the timer with CurrentTime
				GameTimerLED=TimerUpdate(GameTimerLED,CurrentTime);
			}

			//if the GameTimer timer has expired
			if(TimerDone(GameTimer,CurrentTime)) 
			{
				//leave this while loop.
				break;
			}

			//if the Left toggle button is not pushed
			if (InputRead(LeftToggle)==0)
			{
				//update the LeftToggleDebounceTimer with CurrentTime.
				LeftToggleDebounceTimer=TimerUpdate(LeftToggleDebounceTimer,CurrentTime);
				//reset leftlatch.
				leftlatch=0;
			}
			//otherwise, if it is being pushed
			else 
			{
				//if the LeftToggleDebounceTimer has expired and leftlatch is reset
				if(TimerDone(LeftToggleDebounceTimer,CurrentTime)&&(leftlatch==0)) 
				{
					//Shift the prize LEDs left one slot.
					PrizeLEDShiftLeft();
					//set leftlatch to prevent running this if statement from running more than once per button push
					leftlatch=1;
				}
			}
			
			//if the right toggle button is not pushed
			if (InputRead(RightToggle)==0)
			{
				//update the RightToggleDebounceTimer with CurrentTime.
				RightToggleDebounceTimer=TimerUpdate(RightToggleDebounceTimer,CurrentTime);
				//reset rightlatch
				rightlatch=0;
			}
			//otherwise, if it is being pushed
			else 
			{
				//if the RightToggleDebounceTimer has expired and rightlatch is reset
				if(TimerDone(RightToggleDebounceTimer,CurrentTime)&&(rightlatch==0)) 
				{
					//Shift the prize LEDs right one slot.
					PrizeLEDShiftRight();
					//set rightlatch to prevent running this if statement from running more than once per button push
					rightlatch=1;
				}
			}
		}

		//if the soundplay timer did not expire before we left the while loop,
		if(!TimerDone(SoundPlayTimer,CurrentTime))
		{
			//trigger the sound play output to stop the sound.
			OutputWrite(PlaySound,0x00);
			//Wait 100 ms
			Wait(50);
			//set the sound play output high.
			OutputWrite(PlaySound,0xff);
		}

		//execute the following code:
		do
		{
			//set EncoderResult equal to the result of CheckSlot(Encoder)
			EncoderResult=CheckSlot(Encoder);
		}
		//while EncoderResult is zero
		while(EncoderResult==0);

		//if EncoderResult and the Prize LED pattern have overlapping bits, this means the coin dropped in a winning slot.
		if (EncoderResult & PrizeLEDGetPattern())
		{
			//drop swag.
			DropSwag();
		}
	}
}